package nullsafety

fun main(args: Array<String>) {
/*
    // All variables are non-null by default
    var greeting: String = "Hello, World"
    //greeting = null // Compilation Error
    println("The non-nullable greeting: $greeting")


    // Declare nullable variables by appending a `?` in the type declaration
    var nullableGreeting: String? = "Hello, World"
    var nullGreet: String? = "Greetings Earthling"
    nullableGreeting = null // Works
    println("The nullable greeting: $nullableGreeting")

    // Direct method calls and property access on non-null variables are allowed
    val len = greeting.length
    val upper = greeting.toUpperCase()
    println("The len is: $len")
    println("The upper is: $upper")

    //An error is returned calling length method on nullGreet
    //var greetMe = nullGreet.length


    /*
       Direct method calls and property access on nullable variables are no allowed
       because that might result into NullPointerException
       For your exercise: remote the comments from the next 2 lines below

     */

     //Remote the comments
     //val nullableLen = nullableGreeting.length // Compilation Error
     //val nullableUpper = nullableGreeting.toUpperCase()  // Compilation Error


    // ===================================


    // Adding a null check
    var nullableName: String? = "John Null"
    //var nullableName: String? = null

    if (nullableName != null) {
        println("Hello, ${nullableName.toUpperCase()}.")
        println("Your name is ${nullableName.length} characters long.")
    } else {
        println("Hello, Guest")
    }

    // ===================================


    // Using Safe call operator ?.

    println(nullableName?.toUpperCase())


    // ===================================

    // Using ?.let to perform an operation only if a variable is not null
    //  nullableName = null
    nullableName = null

    nullableName?.let { println(it.toUpperCase()) }
    nullableName?.let { println(it.length) }


    //Remove the comments, /* */ on the next 3 lines below
    /* nullableName = "John Null Again"

    nullableName?.let { println(it.toUpperCase()) }
    nullableName?.let { println(it.length) }
    */

    // ===================================


    // Using Elvis operator ?:
    var name = nullableName ?: "Guest"
    val nameLen = nullableName?.length ?: -1

    println("name is $name")
    println("nameLen is $nameLen")
    println(nullableName)

    // ===================================

    nullableName = null
    //nullableName = "Uncle John Null"
    println(nullableName!!.toUpperCase()) // Results in NullPointerException



    // ===================================
*/
    // Collection of nullable types
    val listOfNullableTypes: List<Int?> = listOf(1, 2, null, 3)
    println("Nullable Types:")
    for (nullTypes in listOfNullableTypes){
        print("$nullTypes ")
    }

    //listOfNullableTypes = null
    // ===================================

    // Nullable Collection
    var nullableList: List<Int>? = listOf(1, 2, 3)
    nullableList = null // Works

    println("\nNullable Int List:")
    for (nullIntList in listOfNullableTypes){
        print("$nullIntList ")
    }

    // ===================================

    // Nullable Collection of Nullable Types
    var nullableListOfNullableTypes: List<Int?>? = listOf(1, 2, null, 3) // Works
    nullableListOfNullableTypes = null // Works

    println("\nNullable Int Collection of Types:")
    for (nullListTypes in listOfNullableTypes){
        print("$nullListTypes ")
    }
    print("\n")
}